@@ -3,10 +3,21 @@
 //! This modules contains structures to do with addresses, positions and
 //! suchlike.
 
+#[cfg(feature = "read_write")]
+use crate::read_write::error::FromComponentError;
+#[cfg(feature = "read_write")]
+use crate::read_write::component::Component;
+#[cfg(feature = "read_write")]
+use regex::Regex;
+#[cfg(feature = "read_write")]
+use std::mem;
+
 #[cfg(feature = "diesel")]
 use crate::DieselConversionError;
 
-#[derive(PartialEq)]
+use std::convert::TryFrom;
+
+#[derive(Debug, Clone, PartialEq, PartialOrd)]
 #[non_exhaustive]
 /// A simple structure to hold address information
 pub struct Address {
@@ -29,7 +40,7 @@ pub struct Address {
     pub geo: Option<Geo>,
 }
 
-#[derive(PartialOrd, PartialEq)]
+#[derive(Debug, Clone, PartialEq, PartialOrd)]
 #[non_exhaustive]
 /// A simple structure to hold Geo location.
 pub struct Geo {
@@ -209,3 +220,86 @@ impl AddressRaw {
         }
     }
 }
+
+#[cfg(feature = "read_write")]
+impl From<Address> for Component {
+    fn from(adr: Address) -> Self {
+        Self {
+            // No Group
+            group: None,
+
+            // The parameters will probably be appeneded to later, to add
+            // a type=(work|home), if done through Contact
+            parameters: match adr.geo {
+                Some(org) => {
+                    let mut hashmap = std::collections::HashMap::new();
+                    hashmap.insert(
+                        "geo".to_string(),
+                        format!("{},{}", org.longitude, org.latitude),
+                    );
+                    hashmap
+                }
+                None => std::collections::HashMap::default(),
+            },
+
+            name: "ADR".to_string(),
+
+            values: vec![
+                vec![],
+                vec![],
+                adr.street.map(|x| vec![x]).unwrap_or_default(),
+                adr.locality.map(|x| vec![x]).unwrap_or_default(),
+                adr.region.map(|x| vec![x]).unwrap_or_default(),
+                adr.code.map(|x| vec![x]).unwrap_or_default(),
+                adr.country.map(|x| vec![x]).unwrap_or_default(),
+            ],
+        }
+    }
+}
+
+#[cfg(feature = "read_write")]
+impl TryFrom<Component> for Address {
+    type Error = Box<dyn std::error::Error>;
+
+    fn try_from(mut comp: Component) -> Result<Self, Self::Error> {
+
+        lazy_static!{
+            static ref RE_ORG: Regex = Regex::new(r#"\d+(\.\d+)?,\d+(\.\d+)?"#).unwrap();
+        }
+        
+        // Get rid of extended address and postbox, because, supposedly
+        // they are not longer supported because they are inconsistent.
+        Ok(Self {
+            street: mem::take(&mut comp.values.get_mut(2)).and_then(Vec::pop),
+            locality: mem::take(&mut comp.values.get_mut(3)).and_then(Vec::pop),
+            region: mem::take(&mut comp.values.get_mut(4)).and_then(Vec::pop),
+            code: mem::take(&mut comp.values.get_mut(5)).and_then(Vec::pop),
+            country: mem::take(&mut comp.values.get_mut(6)).and_then(Vec::pop),
+            geo: match comp.parameters.get("GEO") {
+                Some(geo) => {
+                    // Find the GEO part, removing the URI part
+                    let geo = RE_ORG.find(geo).ok_or(FromComponentError::InvalidRegex)?.as_str();
+                    
+                    // Split it by the comma, this should give us the
+                    // long and lat.
+                    let split: Vec<String> =
+                        geo.split(',').map(|x| x.trim().to_owned()).collect();
+
+                    // Checks that it gives two values
+                    if split.len() != 2 {
+                        return Err(Box::new(
+                            FromComponentError::NotEnoughValues,
+                        ));
+                    }
+
+                    // Build the Geo
+                    Some(Geo {
+                        longitude: split[0].parse()?,
+                        latitude: split[1].parse()?,
+                    })
+                }
+                None => None,
+            },
+        })
+    }
+}
