@@ -1,4 +1,13 @@
-#[derive(Eq, PartialEq, Debug)]
+/// A Contact Information is a struct which represents some contact information,
+/// such as an email, phone or online messenger. Currently, there are specifics
+/// for certain messaging programs (based of the ones from
+/// [Evolution](https://wiki.gnome.org/Apps/Evolution)), but adapted to be
+/// a bit more relevant to today.
+///
+/// Use unkown for the vcard property XMPP,
+/// as you can then specify via a URI the messaging service, if you so require
+/// additional services.
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd)]
 #[cfg_attr(
     feature = "diesel-derive-enum",
     derive(DbEnum),
@@ -79,3 +88,9 @@ impl std::fmt::Display for ContactPlatform {
         )
     }
 }
+
+impl Default for ContactPlatform {
+    fn default() -> Self {
+        Self::Unknown
+    }
+}
